import com.typesafe.scalalogging
import java.lang
import scala
import scala.collection
import scala.collection.immutable
import scala.collection.mutable
import scala.runtime
import scala.util
import scorex.crypto.authds.avltree.batch.serialization
import scorex.crypto.hash
import scorex.util
import scorex.util.encode
import typing



class BatchProofConstants:
    @staticmethod
    def $init$($this: 'BatchProofConstants') -> None: ...
    def EndOfTreeInPackagedProof(self) -> int: ...
    def LabelInPackagedProof(self) -> int: ...
    def LeafInPackagedProof(self) -> int: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$EndOfTreeInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$LabelInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$LeafInPackagedProof_$eq(self, x$1: int) -> None: ...

class KeyInVar:
    @staticmethod
    def $init$($this: 'KeyInVar') -> None: ...
    def k(self) -> typing.List[int]: ...
    def k_$eq(self, x$1: typing.List[int]) -> None: ...
    def key(self) -> typing.List[int]: ...

class Operation:
    def key(self) -> typing.List[int]: ...

_PersistentBatchAVLProver__D = typing.TypeVar('_PersistentBatchAVLProver__D', bound=typing.List[int])  # <D>
_PersistentBatchAVLProver__HF = typing.TypeVar('_PersistentBatchAVLProver__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
class PersistentBatchAVLProver(typing.Generic[_PersistentBatchAVLProver__D, _PersistentBatchAVLProver__HF]):
    def __init__(self): ...
    def avlProver(self) -> 'BatchAVLProver'[_PersistentBatchAVLProver__D, _PersistentBatchAVLProver__HF]: ...
    def avlProver_$eq(self, x$1: 'BatchAVLProver'[_PersistentBatchAVLProver__D, _PersistentBatchAVLProver__HF]) -> None: ...
    def checkTree(self, postProof: bool) -> None: ...
    def checkTree$default$1(self) -> bool: ...
    _create_0__D = typing.TypeVar('_create_0__D', bound=typing.List[int])  # <D>
    _create_0__HF = typing.TypeVar('_create_0__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    _create_1__D = typing.TypeVar('_create_1__D', bound=typing.List[int])  # <D>
    _create_1__HF = typing.TypeVar('_create_1__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    _create_1__K = typing.TypeVar('_create_1__K', bound=typing.List[int])  # <K>
    _create_1__V = typing.TypeVar('_create_1__V', bound=typing.List[int])  # <V>
    @typing.overload
    @staticmethod
    def create(avlBatchProver: 'BatchAVLProver'[_create_0__D, _create_0__HF], versionedStorage: 'VersionedAVLStorage'[_create_0__D], paranoidChecks: bool) -> scala.util.Try['PersistentBatchAVLProver'[_create_0__D, _create_0__HF]]: ...
    @typing.overload
    @staticmethod
    def create(avlBatchProver: 'BatchAVLProver'[_create_1__D, _create_1__HF], versionedStorage: 'VersionedAVLStorage'[_create_1__D], additionalData: scala.collection.Seq[scala.Tuple2[_create_1__K, _create_1__V]], paranoidChecks: bool) -> scala.util.Try['PersistentBatchAVLProver'[_create_1__D, _create_1__HF]]: ...
    _create$default$3__D = typing.TypeVar('_create$default$3__D', bound=typing.List[int])  # <D>
    _create$default$3__HF = typing.TypeVar('_create$default$3__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def create$default$3() -> bool: ...
    def digest(self) -> typing.List[int]: ...
    _generateProofAndUpdateStorage_1__K = typing.TypeVar('_generateProofAndUpdateStorage_1__K', bound=typing.List[int])  # <K>
    _generateProofAndUpdateStorage_1__V = typing.TypeVar('_generateProofAndUpdateStorage_1__V', bound=typing.List[int])  # <V>
    @typing.overload
    def generateProofAndUpdateStorage(self) -> typing.List[int]: ...
    @typing.overload
    def generateProofAndUpdateStorage(self, additionalData: scala.collection.Seq[scala.Tuple2[_generateProofAndUpdateStorage_1__K, _generateProofAndUpdateStorage_1__V]]) -> typing.List[int]: ...
    def height(self) -> int: ...
    def performOneOperation(self, operation: Operation) -> scala.util.Try[scala.Option[typing.List[int]]]: ...
    def prover(self) -> 'BatchAVLProver'[_PersistentBatchAVLProver__D, _PersistentBatchAVLProver__HF]: ...
    def rollback(self, version: typing.List[int]) -> scala.util.Try[scala.runtime.BoxedUnit]: ...
    def storage(self) -> 'VersionedAVLStorage'[_PersistentBatchAVLProver__D]: ...
    def unauthenticatedLookup(self, key: typing.List[int]) -> scala.Option[typing.List[int]]: ...

class ToStringHelper(scorex.util.ScorexEncoding):
    @staticmethod
    def $init$($this: 'ToStringHelper') -> None: ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...

class UnknownModification:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def encoder() -> scorex.util.encode.BytesEncoder: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def key() -> typing.List[int]: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def toString() -> java.lang.String: ...
    @staticmethod
    def updateFn() -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...

_VersionedAVLStorage__D = typing.TypeVar('_VersionedAVLStorage__D', bound=typing.List[int])  # <D>
class VersionedAVLStorage(typing.Generic[_VersionedAVLStorage__D]):
    @staticmethod
    def $init$($this: 'VersionedAVLStorage') -> None: ...
    def isEmpty(self) -> bool: ...
    def nonEmpty(self) -> bool: ...
    def rollback(self, version: typing.List[int]) -> scala.util.Try[scala.Tuple2['ProverNodes'[_VersionedAVLStorage__D], typing.Any]]: ...
    def rollbackVersions(self) -> scala.collection.Iterable[typing.List[int]]: ...
    _update_0__K = typing.TypeVar('_update_0__K', bound=typing.List[int])  # <K>
    _update_0__V = typing.TypeVar('_update_0__V', bound=typing.List[int])  # <V>
    @typing.overload
    def update(self, batchProver: 'BatchAVLProver'[_VersionedAVLStorage__D, typing.Any], additionalData: scala.collection.Seq[scala.Tuple2[_update_0__K, _update_0__V]]) -> scala.util.Try[scala.runtime.BoxedUnit]: ...
    @typing.overload
    def update(self, batchProver: 'BatchAVLProver'[_VersionedAVLStorage__D, typing.Any]) -> scala.util.Try[scala.runtime.BoxedUnit]: ...
    def version(self) -> scala.Option[typing.List[int]]: ...

_AuthenticatedTreeOps__D = typing.TypeVar('_AuthenticatedTreeOps__D', bound=typing.List[int])  # <D>
class AuthenticatedTreeOps(BatchProofConstants, ToStringHelper, typing.Generic[_AuthenticatedTreeOps__D]):
    @staticmethod
    def $init$($this: 'AuthenticatedTreeOps') -> None: ...
    def NegativeInfinityKey(self) -> typing.List[int]: ...
    def PositiveInfinityKey(self) -> typing.List[int]: ...
    def addNode(self, r: 'Leaf'[_AuthenticatedTreeOps__D], key: typing.List[int], v: typing.List[int]) -> 'InternalNode'[_AuthenticatedTreeOps__D]: ...
    def changedNodesBuffer(self) -> scala.collection.mutable.ArrayBuffer['ProverNodes'[_AuthenticatedTreeOps__D]]: ...
    def changedNodesBufferToCheck(self) -> scala.collection.mutable.ArrayBuffer['ProverNodes'[_AuthenticatedTreeOps__D]]: ...
    def collectChangedNodes(self) -> bool: ...
    def digest(self, rootNode: 'Node'[_AuthenticatedTreeOps__D]) -> typing.List[int]: ...
    def keyLength(self) -> int: ...
    def keyMatchesLeaf(self, key: typing.List[int], r: 'Leaf'[_AuthenticatedTreeOps__D]) -> bool: ...
    def nextDirectionIsLeft(self, key: typing.List[int], r: 'InternalNode'[_AuthenticatedTreeOps__D]) -> bool: ...
    def onNodeVisit(self, n: 'Node'[_AuthenticatedTreeOps__D], operation: Operation, isRotate: bool) -> None: ...
    def onNodeVisit$default$3(self) -> bool: ...
    def replayComparison(self) -> int: ...
    def returnResultOfOneOperation(self, operation: Operation, rootNode: 'Node'[_AuthenticatedTreeOps__D]) -> scala.util.Try[scala.Tuple2['Node'[_AuthenticatedTreeOps__D], scala.Option[typing.List[int]]]]: ...
    def rootNodeHeight(self) -> int: ...
    def rootNodeHeight_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$NegativeInfinityKey_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$PositiveInfinityKey_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$changedNodesBufferToCheck_$eq(self, x$1: scala.collection.mutable.ArrayBuffer['ProverNodes'[_AuthenticatedTreeOps__D]]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$changedNodesBuffer_$eq(self, x$1: scala.collection.mutable.ArrayBuffer['ProverNodes'[_AuthenticatedTreeOps__D]]) -> None: ...
    def valueLengthOpt(self) -> scala.Option[typing.Any]: ...

class Lookup(Operation, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['Lookup', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(key: typing.List[int]) -> 'Lookup': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'Lookup']: ...
    def copy(self, key: typing.List[int]) -> 'Lookup': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'Lookup') -> scala.Option[typing.List[int]]: ...

class Modification(Operation, scorex.util.ScorexEncoding):
    def key(self) -> typing.List[int]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...

_Node__D = typing.TypeVar('_Node__D', bound=typing.List[int])  # <D>
class Node(ToStringHelper, typing.Generic[_Node__D]):
    @staticmethod
    def $init$($this: 'Node') -> None: ...
    def computeLabel(self) -> _Node__D: ...
    def label(self) -> _Node__D: ...
    def labelOpt(self) -> scala.Option[_Node__D]: ...
    def labelOpt_$eq(self, x$1: scala.Option[_Node__D]) -> None: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

_BatchAVLProver__D = typing.TypeVar('_BatchAVLProver__D', bound=typing.List[int])  # <D>
_BatchAVLProver__HF = typing.TypeVar('_BatchAVLProver__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
class BatchAVLProver(AuthenticatedTreeOps[_BatchAVLProver__D], scorex.util.ScorexLogging, typing.Generic[_BatchAVLProver__D, _BatchAVLProver__HF]):
    def __init__(self, keyLength: int, valueLengthOpt: scala.Option[typing.Any], oldRootAndHeight: scala.Option[scala.Tuple2['ProverNodes'[_BatchAVLProver__D], typing.Any]], collectChangedNodes: bool, hf: _BatchAVLProver__HF): ...
    _$lessinit$greater$default$3__D = typing.TypeVar('_$lessinit$greater$default$3__D', bound=typing.List[int])  # <D>
    _$lessinit$greater$default$3__HF = typing.TypeVar('_$lessinit$greater$default$3__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$3() -> scala.None.: ...
    _$lessinit$greater$default$4__D = typing.TypeVar('_$lessinit$greater$default$4__D', bound=typing.List[int])  # <D>
    _$lessinit$greater$default$4__HF = typing.TypeVar('_$lessinit$greater$default$4__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$4() -> bool: ...
    _$lessinit$greater$default$5__D = typing.TypeVar('_$lessinit$greater$default$5__D', bound=typing.List[int])  # <D>
    _$lessinit$greater$default$5__HF = typing.TypeVar('_$lessinit$greater$default$5__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$5(keyLength: int, valueLengthOpt: scala.Option[typing.Any], oldRootAndHeight: scala.Option[scala.Tuple2['ProverNodes'[_.lessinit.greater.default.5__D], typing.Any]], collectChangedNodes: bool) -> scorex.crypto.hash.Blake2b256.: ...
    def EndOfTreeInPackagedProof(self) -> int: ...
    def LabelInPackagedProof(self) -> int: ...
    def LeafInPackagedProof(self) -> int: ...
    def NegativeInfinityKey(self) -> typing.List[int]: ...
    def PositiveInfinityKey(self) -> typing.List[int]: ...
    def addNode(self, r: 'Leaf'[_BatchAVLProver__D], key: typing.List[int], v: typing.List[int]) -> 'InternalProverNode'[_BatchAVLProver__D]: ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def changedNodesBuffer(self) -> scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLProver__D]]: ...
    def changedNodesBufferToCheck(self) -> scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLProver__D]]: ...
    def checkTree(self, postProof: bool) -> None: ...
    def checkTree$default$1(self) -> bool: ...
    def collectChangedNodes(self) -> bool: ...
    def com$typesafe$scalalogging$StrictLogging$_setter_$logger_$eq(self, x$1: com.typesafe.scalalogging.Logger) -> None: ...
    @typing.overload
    def contains(self, key: typing.List[int], label: _BatchAVLProver__D) -> bool: ...
    @typing.overload
    def contains(self, node: 'ProverNodes'[_BatchAVLProver__D]) -> bool: ...
    @typing.overload
    def digest(self) -> typing.List[int]: ...
    @typing.overload
    def digest(self, rootNode: Node[_BatchAVLProver__D]) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def generateProof(self) -> typing.List[int]: ...
    def generateProofForOperations(self, operations: scala.collection.Seq[Operation]) -> scala.util.Try[scala.Tuple2[typing.List[int], typing.List[int]]]: ...
    def hf(self) -> _BatchAVLProver__HF: ...
    def keyLength(self) -> int: ...
    def keyMatchesLeaf(self, key: typing.List[int], r: 'Leaf'[_BatchAVLProver__D]) -> bool: ...
    def labelLength(self) -> int: ...
    def log(self) -> com.typesafe.scalalogging.Logger: ...
    def logger(self) -> com.typesafe.scalalogging.Logger: ...
    def nextDirectionIsLeft(self, key: typing.List[int], r: 'InternalNode'[_BatchAVLProver__D]) -> bool: ...
    def onNodeVisit(self, n: Node[_BatchAVLProver__D], operation: Operation, isRotate: bool) -> None: ...
    def onNodeVisit$default$3(self) -> bool: ...
    def performOneOperation(self, operation: Operation) -> scala.util.Try[scala.Option[typing.List[int]]]: ...
    def randomWalk(self, rand: scala.util.Random) -> scala.Option[scala.Tuple2[typing.List[int], typing.List[int]]]: ...
    def randomWalk$default$1(self) -> scala.util.Random: ...
    def removedNodes(self) -> scala.collection.immutable.List['ProverNodes'[_BatchAVLProver__D]]: ...
    def replayComparison(self) -> int: ...
    def returnResultOfOneOperation(self, operation: Operation, rootNode: Node[_BatchAVLProver__D]) -> scala.util.Try[scala.Tuple2[Node[_BatchAVLProver__D], scala.Option[typing.List[int]]]]: ...
    def rootNodeHeight(self) -> int: ...
    def rootNodeHeight_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$NegativeInfinityKey_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$PositiveInfinityKey_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$changedNodesBufferToCheck_$eq(self, x$1: scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLProver__D]]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$changedNodesBuffer_$eq(self, x$1: scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLProver__D]]) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$EndOfTreeInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$LabelInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$LeafInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    def topNode(self) -> 'ProverNodes'[_BatchAVLProver__D]: ...
    def topNode_$eq(self, x$1: 'ProverNodes'[_BatchAVLProver__D]) -> None: ...
    _treeWalk__IR = typing.TypeVar('_treeWalk__IR')  # <IR>
    _treeWalk__LR = typing.TypeVar('_treeWalk__LR')  # <LR>
    def treeWalk(self, internalNodeFn: scala.Function2['InternalProverNode'[_BatchAVLProver__D], _treeWalk__IR, scala.Tuple2['ProverNodes'[_BatchAVLProver__D], _treeWalk__IR]], leafFn: scala.Function2['ProverLeaf'[_BatchAVLProver__D], _treeWalk__IR, _treeWalk__LR], initial: _treeWalk__IR) -> _treeWalk__LR: ...
    def unauthenticatedLookup(self, key: typing.List[int]) -> scala.Option[typing.List[int]]: ...
    def valueLengthOpt(self) -> scala.Option[typing.Any]: ...

_BatchAVLVerifier__D = typing.TypeVar('_BatchAVLVerifier__D', bound=typing.List[int])  # <D>
_BatchAVLVerifier__HF = typing.TypeVar('_BatchAVLVerifier__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
class BatchAVLVerifier(AuthenticatedTreeOps[_BatchAVLVerifier__D], typing.Generic[_BatchAVLVerifier__D, _BatchAVLVerifier__HF]):
    def __init__(self, startingDigest: typing.List[int], proof: typing.List[int], keyLength: int, valueLengthOpt: scala.Option[typing.Any], maxNumOperations: scala.Option[typing.Any], maxDeletes: scala.Option[typing.Any], hf: _BatchAVLVerifier__HF): ...
    _$lessinit$greater$default$5__D = typing.TypeVar('_$lessinit$greater$default$5__D', bound=typing.List[int])  # <D>
    _$lessinit$greater$default$5__HF = typing.TypeVar('_$lessinit$greater$default$5__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$5() -> scala.Option[typing.Any]: ...
    _$lessinit$greater$default$6__D = typing.TypeVar('_$lessinit$greater$default$6__D', bound=typing.List[int])  # <D>
    _$lessinit$greater$default$6__HF = typing.TypeVar('_$lessinit$greater$default$6__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$6() -> scala.Option[typing.Any]: ...
    _$lessinit$greater$default$7__D = typing.TypeVar('_$lessinit$greater$default$7__D', bound=typing.List[int])  # <D>
    _$lessinit$greater$default$7__HF = typing.TypeVar('_$lessinit$greater$default$7__HF', bound=scorex.crypto.hash.CryptographicHash)  # <HF>
    @staticmethod
    def $lessinit$greater$default$7(startingDigest: typing.List[int], proof: typing.List[int], keyLength: int, valueLengthOpt: scala.Option[typing.Any], maxNumOperations: scala.Option[typing.Any], maxDeletes: scala.Option[typing.Any]) -> scorex.crypto.hash.Blake2b256.: ...
    def EndOfTreeInPackagedProof(self) -> int: ...
    def LabelInPackagedProof(self) -> int: ...
    def LeafInPackagedProof(self) -> int: ...
    def NegativeInfinityKey(self) -> typing.List[int]: ...
    def PositiveInfinityKey(self) -> typing.List[int]: ...
    def addNode(self, r: 'Leaf'[_BatchAVLVerifier__D], key: typing.List[int], v: typing.List[int]) -> 'InternalVerifierNode'[_BatchAVLVerifier__D]: ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def changedNodesBuffer(self) -> scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLVerifier__D]]: ...
    def changedNodesBufferToCheck(self) -> scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLVerifier__D]]: ...
    def collectChangedNodes(self) -> bool: ...
    @typing.overload
    def digest(self, rootNode: Node[_BatchAVLVerifier__D]) -> typing.List[int]: ...
    @typing.overload
    def digest(self) -> scala.Option[typing.List[int]]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def extractFirstNode(self, extractor: scala.Function1['VerifierNodes'[_BatchAVLVerifier__D], typing.Any]) -> scala.Option['VerifierNodes'[_BatchAVLVerifier__D]]: ...
    def extractNodes(self, extractor: scala.Function1['VerifierNodes'[_BatchAVLVerifier__D], typing.Any]) -> scala.Option[scala.collection.Seq['VerifierNodes'[_BatchAVLVerifier__D]]]: ...
    def keyLength(self) -> int: ...
    def keyMatchesLeaf(self, key: typing.List[int], r: 'Leaf'[_BatchAVLVerifier__D]) -> bool: ...
    def labelLength(self) -> int: ...
    def logError(self, t: java.lang.Throwable) -> None: ...
    def nextDirectionIsLeft(self, key: typing.List[int], r: 'InternalNode'[_BatchAVLVerifier__D]) -> bool: ...
    def onNodeVisit(self, n: Node[_BatchAVLVerifier__D], operation: Operation, isRotate: bool) -> None: ...
    def onNodeVisit$default$3(self) -> bool: ...
    def performOneOperation(self, operation: Operation) -> scala.util.Try[scala.Option[typing.List[int]]]: ...
    def replayComparison(self) -> int: ...
    def returnResultOfOneOperation(self, operation: Operation, rootNode: Node[_BatchAVLVerifier__D]) -> scala.util.Try[scala.Tuple2[Node[_BatchAVLVerifier__D], scala.Option[typing.List[int]]]]: ...
    def rootNodeHeight(self) -> int: ...
    def rootNodeHeight_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$NegativeInfinityKey_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$PositiveInfinityKey_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$changedNodesBufferToCheck_$eq(self, x$1: scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLVerifier__D]]) -> None: ...
    def scorex$crypto$authds$avltree$batch$AuthenticatedTreeOps$_setter_$changedNodesBuffer_$eq(self, x$1: scala.collection.mutable.ArrayBuffer['ProverNodes'[_BatchAVLVerifier__D]]) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$EndOfTreeInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$LabelInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$crypto$authds$avltree$batch$BatchProofConstants$_setter_$LeafInPackagedProof_$eq(self, x$1: int) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    def valueLengthOpt(self) -> scala.Option[typing.Any]: ...

class Insert(Modification, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], value: typing.List[int]): ...
    @staticmethod
    def apply(key: typing.List[int], value: typing.List[int]) -> 'Insert': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, key: typing.List[int], value: typing.List[int]) -> 'Insert': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> typing.List[int]: ...
    @staticmethod
    def curried() -> scala.Function1[typing.List[int], scala.Function1[typing.List[int], 'Insert']]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[typing.List[int], typing.List[int]], 'Insert']: ...
    @staticmethod
    def unapply(x$0: 'Insert') -> scala.Option[scala.Tuple2[typing.List[int], typing.List[int]]]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...
    def value(self) -> typing.List[int]: ...

class InsertOrUpdate(Modification, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], value: typing.List[int]): ...
    @staticmethod
    def apply(key: typing.List[int], value: typing.List[int]) -> 'InsertOrUpdate': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, key: typing.List[int], value: typing.List[int]) -> 'InsertOrUpdate': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> typing.List[int]: ...
    @staticmethod
    def curried() -> scala.Function1[typing.List[int], scala.Function1[typing.List[int], 'InsertOrUpdate']]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[typing.List[int], typing.List[int]], 'InsertOrUpdate']: ...
    @staticmethod
    def unapply(x$0: 'InsertOrUpdate') -> scala.Option[scala.Tuple2[typing.List[int], typing.List[int]]]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...
    def value(self) -> typing.List[int]: ...

_InternalNode__D = typing.TypeVar('_InternalNode__D', bound=typing.List[int])  # <D>
class InternalNode(Node[_InternalNode__D], typing.Generic[_InternalNode__D]):
    @staticmethod
    def $init$($this: 'InternalNode') -> None: ...
    def b(self) -> int: ...
    def b_$eq(self, x$1: int) -> None: ...
    def balance(self) -> int: ...
    def computeLabel(self) -> _InternalNode__D: ...
    def getNew(self, newLeft: Node[_InternalNode__D], newRight: Node[_InternalNode__D], newBalance: int) -> 'InternalNode'[_InternalNode__D]: ...
    def getNew$default$1(self) -> Node[_InternalNode__D]: ...
    def getNew$default$2(self) -> Node[_InternalNode__D]: ...
    def getNew$default$3(self) -> int: ...
    def getNewKey(self, newKey: typing.List[int]) -> 'InternalNode'[_InternalNode__D]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_InternalNode__D]: ...
    def left(self) -> Node[_InternalNode__D]: ...
    def right(self) -> Node[_InternalNode__D]: ...

_Leaf__D = typing.TypeVar('_Leaf__D', bound=typing.List[int])  # <D>
class Leaf(Node[_Leaf__D], KeyInVar, typing.Generic[_Leaf__D]):
    @staticmethod
    def $init$($this: 'Leaf') -> None: ...
    def computeLabel(self) -> _Leaf__D: ...
    def getNew(self, newKey: typing.List[int], newValue: typing.List[int], newNextLeafKey: typing.List[int]) -> 'Leaf'[_Leaf__D]: ...
    def getNew$default$1(self) -> typing.List[int]: ...
    def getNew$default$2(self) -> typing.List[int]: ...
    def getNew$default$3(self) -> typing.List[int]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_Leaf__D]: ...
    def nextLeafKey(self) -> typing.List[int]: ...
    def nk(self) -> typing.List[int]: ...
    def nk_$eq(self, x$1: typing.List[int]) -> None: ...
    def toString(self) -> java.lang.String: ...
    def v(self) -> typing.List[int]: ...
    def v_$eq(self, x$1: typing.List[int]) -> None: ...
    def value(self) -> typing.List[int]: ...

_ProverNodes__D = typing.TypeVar('_ProverNodes__D', bound=typing.List[int])  # <D>
class ProverNodes(Node[_ProverNodes__D], KeyInVar, typing.Generic[_ProverNodes__D]):
    @staticmethod
    def $init$($this: 'ProverNodes') -> None: ...
    def isNew(self) -> bool: ...
    def isNew_$eq(self, x$1: bool) -> None: ...

class Remove(Modification, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['Remove', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(key: typing.List[int]) -> 'Remove': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'Remove']: ...
    def copy(self, key: typing.List[int]) -> 'Remove': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'Remove') -> scala.Option[typing.List[int]]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...

class RemoveIfExists(Modification, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['RemoveIfExists', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(key: typing.List[int]) -> 'RemoveIfExists': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'RemoveIfExists']: ...
    def copy(self, key: typing.List[int]) -> 'RemoveIfExists': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'RemoveIfExists') -> scala.Option[typing.List[int]]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...

class Update(Modification, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], value: typing.List[int]): ...
    @staticmethod
    def apply(key: typing.List[int], value: typing.List[int]) -> 'Update': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, key: typing.List[int], value: typing.List[int]) -> 'Update': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> typing.List[int]: ...
    @staticmethod
    def curried() -> scala.Function1[typing.List[int], scala.Function1[typing.List[int], 'Update']]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[typing.List[int], typing.List[int]], 'Update']: ...
    @staticmethod
    def unapply(x$0: 'Update') -> scala.Option[scala.Tuple2[typing.List[int], typing.List[int]]]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...
    def value(self) -> typing.List[int]: ...

class UpdateLongBy(Modification, scala.Product, scala.Serializable):
    def __init__(self, key: typing.List[int], delta: int): ...
    @staticmethod
    def apply(key: typing.List[int], delta: int) -> 'UpdateLongBy': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, key: typing.List[int], delta: int) -> 'UpdateLongBy': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def copy$default$2(self) -> int: ...
    @staticmethod
    def curried() -> scala.Function1[typing.List[int], scala.Function1[typing.Any, 'UpdateLongBy']]: ...
    def delta(self) -> int: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def key(self) -> typing.List[int]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[typing.List[int], typing.Any], 'UpdateLongBy']: ...
    @staticmethod
    def unapply(x$0: 'UpdateLongBy') -> scala.Option[scala.Tuple2[typing.List[int], typing.Any]]: ...
    def updateFn(self) -> scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]: ...

_VerifierNodes__D = typing.TypeVar('_VerifierNodes__D', bound=typing.List[int])  # <D>
class VerifierNodes(Node[_VerifierNodes__D], typing.Generic[_VerifierNodes__D]): ...

_InternalProverNode__D = typing.TypeVar('_InternalProverNode__D', bound=typing.List[int])  # <D>
class InternalProverNode(ProverNodes[_InternalProverNode__D], InternalNode[_InternalProverNode__D], typing.Generic[_InternalProverNode__D]):
    def __init__(self, k: typing.List[int], l: ProverNodes[_InternalProverNode__D], r: ProverNodes[_InternalProverNode__D], b: int, hf: scorex.crypto.hash.CryptographicHash[_InternalProverNode__D]): ...
    _$lessinit$greater$default$4__D = typing.TypeVar('_$lessinit$greater$default$4__D', bound=typing.List[int])  # <D>
    @staticmethod
    def $lessinit$greater$default$4() -> int: ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def b(self) -> int: ...
    def b_$eq(self, x$1: int) -> None: ...
    def balance(self) -> int: ...
    def computeLabel(self) -> _InternalProverNode__D: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def getNew(self, newLeft: Node[_InternalProverNode__D], newRight: Node[_InternalProverNode__D], newBalance: int) -> 'InternalProverNode'[_InternalProverNode__D]: ...
    def getNew$default$1(self) -> Node[_InternalProverNode__D]: ...
    def getNew$default$2(self) -> Node[_InternalProverNode__D]: ...
    def getNew$default$3(self) -> int: ...
    def getNewKey(self, newKey: typing.List[int]) -> 'InternalProverNode'[_InternalProverNode__D]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_InternalProverNode__D]: ...
    def isNew(self) -> bool: ...
    def isNew_$eq(self, x$1: bool) -> None: ...
    def k(self) -> typing.List[int]: ...
    def k_$eq(self, x$1: typing.List[int]) -> None: ...
    def key(self) -> typing.List[int]: ...
    def l(self) -> ProverNodes[_InternalProverNode__D]: ...
    def l_$eq(self, x$1: ProverNodes[_InternalProverNode__D]) -> None: ...
    def label(self) -> _InternalProverNode__D: ...
    def labelOpt(self) -> scala.Option[_InternalProverNode__D]: ...
    def labelOpt_$eq(self, x$1: scala.Option[_InternalProverNode__D]) -> None: ...
    def left(self) -> ProverNodes[_InternalProverNode__D]: ...
    def r(self) -> ProverNodes[_InternalProverNode__D]: ...
    def r_$eq(self, x$1: ProverNodes[_InternalProverNode__D]) -> None: ...
    def right(self) -> ProverNodes[_InternalProverNode__D]: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

_InternalVerifierNode__D = typing.TypeVar('_InternalVerifierNode__D', bound=typing.List[int])  # <D>
class InternalVerifierNode(VerifierNodes[_InternalVerifierNode__D], InternalNode[_InternalVerifierNode__D], typing.Generic[_InternalVerifierNode__D]):
    def __init__(self, l: Node[_InternalVerifierNode__D], r: Node[_InternalVerifierNode__D], b: int, hf: scorex.crypto.hash.CryptographicHash[_InternalVerifierNode__D]): ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def b(self) -> int: ...
    def b_$eq(self, x$1: int) -> None: ...
    def balance(self) -> int: ...
    def computeLabel(self) -> _InternalVerifierNode__D: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def getNew(self, newLeft: Node[_InternalVerifierNode__D], newRight: Node[_InternalVerifierNode__D], newBalance: int) -> 'InternalVerifierNode'[_InternalVerifierNode__D]: ...
    def getNew$default$1(self) -> Node[_InternalVerifierNode__D]: ...
    def getNew$default$2(self) -> Node[_InternalVerifierNode__D]: ...
    def getNew$default$3(self) -> int: ...
    def getNewKey(self, newKey: typing.List[int]) -> InternalNode[_InternalVerifierNode__D]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_InternalVerifierNode__D]: ...
    def l(self) -> Node[_InternalVerifierNode__D]: ...
    def l_$eq(self, x$1: Node[_InternalVerifierNode__D]) -> None: ...
    def label(self) -> _InternalVerifierNode__D: ...
    def labelOpt(self) -> scala.Option[_InternalVerifierNode__D]: ...
    def labelOpt_$eq(self, x$1: scala.Option[_InternalVerifierNode__D]) -> None: ...
    def left(self) -> Node[_InternalVerifierNode__D]: ...
    def r(self) -> Node[_InternalVerifierNode__D]: ...
    def r_$eq(self, x$1: Node[_InternalVerifierNode__D]) -> None: ...
    def right(self) -> Node[_InternalVerifierNode__D]: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

_LabelOnlyNode__D = typing.TypeVar('_LabelOnlyNode__D', bound=typing.List[int])  # <D>
class LabelOnlyNode(VerifierNodes[_LabelOnlyNode__D], typing.Generic[_LabelOnlyNode__D]):
    def __init__(self, l: _LabelOnlyNode__D): ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def computeLabel(self) -> _LabelOnlyNode__D: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def label(self) -> _LabelOnlyNode__D: ...
    def labelOpt(self) -> scala.Option[_LabelOnlyNode__D]: ...
    def labelOpt_$eq(self, x$1: scala.Option[_LabelOnlyNode__D]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

_ProverLeaf__D = typing.TypeVar('_ProverLeaf__D', bound=typing.List[int])  # <D>
class ProverLeaf(Leaf[_ProverLeaf__D], ProverNodes[_ProverLeaf__D], typing.Generic[_ProverLeaf__D]):
    def __init__(self, k: typing.List[int], v: typing.List[int], nk: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[_ProverLeaf__D]): ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def computeLabel(self) -> _ProverLeaf__D: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def getNew(self, newKey: typing.List[int], newValue: typing.List[int], newNextLeafKey: typing.List[int]) -> 'ProverLeaf'[_ProverLeaf__D]: ...
    def getNew$default$1(self) -> typing.List[int]: ...
    def getNew$default$2(self) -> typing.List[int]: ...
    def getNew$default$3(self) -> typing.List[int]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_ProverLeaf__D]: ...
    def isNew(self) -> bool: ...
    def isNew_$eq(self, x$1: bool) -> None: ...
    def k(self) -> typing.List[int]: ...
    def k_$eq(self, x$1: typing.List[int]) -> None: ...
    def key(self) -> typing.List[int]: ...
    def label(self) -> _ProverLeaf__D: ...
    def labelOpt(self) -> scala.Option[_ProverLeaf__D]: ...
    def labelOpt_$eq(self, x$1: scala.Option[_ProverLeaf__D]) -> None: ...
    def nextLeafKey(self) -> typing.List[int]: ...
    def nk(self) -> typing.List[int]: ...
    def nk_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    def v(self) -> typing.List[int]: ...
    def v_$eq(self, x$1: typing.List[int]) -> None: ...
    def value(self) -> typing.List[int]: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...

_VerifierLeaf__D = typing.TypeVar('_VerifierLeaf__D', bound=typing.List[int])  # <D>
class VerifierLeaf(Leaf[_VerifierLeaf__D], VerifierNodes[_VerifierLeaf__D], typing.Generic[_VerifierLeaf__D]):
    def __init__(self, k: typing.List[int], v: typing.List[int], nk: typing.List[int], hf: scorex.crypto.hash.CryptographicHash[_VerifierLeaf__D]): ...
    def arrayToString(self, a: typing.List[int]) -> java.lang.String: ...
    def computeLabel(self) -> _VerifierLeaf__D: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def getNew(self, newKey: typing.List[int], newValue: typing.List[int], newNextLeafKey: typing.List[int]) -> 'VerifierLeaf'[_VerifierLeaf__D]: ...
    def getNew$default$1(self) -> typing.List[int]: ...
    def getNew$default$2(self) -> typing.List[int]: ...
    def getNew$default$3(self) -> typing.List[int]: ...
    def hf(self) -> scorex.crypto.hash.CryptographicHash[_VerifierLeaf__D]: ...
    def k(self) -> typing.List[int]: ...
    def k_$eq(self, x$1: typing.List[int]) -> None: ...
    def key(self) -> typing.List[int]: ...
    def label(self) -> _VerifierLeaf__D: ...
    def labelOpt(self) -> scala.Option[_VerifierLeaf__D]: ...
    def labelOpt_$eq(self, x$1: scala.Option[_VerifierLeaf__D]) -> None: ...
    def nextLeafKey(self) -> typing.List[int]: ...
    def nk(self) -> typing.List[int]: ...
    def nk_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    def v(self) -> typing.List[int]: ...
    def v_$eq(self, x$1: typing.List[int]) -> None: ...
    def value(self) -> typing.List[int]: ...
    def visited(self) -> bool: ...
    def visited_$eq(self, x$1: bool) -> None: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto.authds.avltree.batch")``.

    AuthenticatedTreeOps: typing.Type[AuthenticatedTreeOps]
    BatchAVLProver: typing.Type[BatchAVLProver]
    BatchAVLVerifier: typing.Type[BatchAVLVerifier]
    BatchProofConstants: typing.Type[BatchProofConstants]
    Insert: typing.Type[Insert]
    InsertOrUpdate: typing.Type[InsertOrUpdate]
    InternalNode: typing.Type[InternalNode]
    InternalProverNode: typing.Type[InternalProverNode]
    InternalVerifierNode: typing.Type[InternalVerifierNode]
    KeyInVar: typing.Type[KeyInVar]
    LabelOnlyNode: typing.Type[LabelOnlyNode]
    Leaf: typing.Type[Leaf]
    Lookup: typing.Type[Lookup]
    Modification: typing.Type[Modification]
    Node: typing.Type[Node]
    Operation: typing.Type[Operation]
    PersistentBatchAVLProver: typing.Type[PersistentBatchAVLProver]
    ProverLeaf: typing.Type[ProverLeaf]
    ProverNodes: typing.Type[ProverNodes]
    Remove: typing.Type[Remove]
    RemoveIfExists: typing.Type[RemoveIfExists]
    ToStringHelper: typing.Type[ToStringHelper]
    UnknownModification: typing.Type[UnknownModification]
    Update: typing.Type[Update]
    UpdateLongBy: typing.Type[UpdateLongBy]
    VerifierLeaf: typing.Type[VerifierLeaf]
    VerifierNodes: typing.Type[VerifierNodes]
    VersionedAVLStorage: typing.Type[VersionedAVLStorage]
    serialization: scorex.crypto.authds.avltree.batch.serialization.__module_protocol__



import java.lang
import java.math
import java.security
import org.bouncycastle.asn1.x9
import org.bouncycastle.math.ec
import org.bouncycastle.math.ec.custom.sec
import scala
import scala.collection
import scala.concurrent
import scala.math
import scala.runtime
import sigmastate
import special.sigma
import supertagged
import typing



class DiffieHellmanTupleInteractiveProver:
    @staticmethod
    def computeCommitment(proposition: 'ProveDHTuple', challenge: typing.List[int], secondMessage: 'SecondDiffieHellmanTupleProverMessage') -> scala.Tuple2[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, org.bouncycastle.math.ec.custom.sec.SecP256K1Point]: ...
    @staticmethod
    def firstMessage(publicInput: 'ProveDHTuple') -> scala.Tuple2[java.math.BigInteger, 'FirstDiffieHellmanTupleProverMessage']: ...
    @staticmethod
    def secondMessage(privateInput: 'DiffieHellmanTupleProverInput', rnd: java.math.BigInteger, challenge: typing.List[int]) -> 'SecondDiffieHellmanTupleProverMessage': ...
    @staticmethod
    def simulate(publicInput: 'ProveDHTuple', challenge: typing.List[int]) -> scala.Tuple2['FirstDiffieHellmanTupleProverMessage', 'SecondDiffieHellmanTupleProverMessage']: ...

_DlogGroup__ElemType = typing.TypeVar('_DlogGroup__ElemType', bound=org.bouncycastle.math.ec.ECPoint)  # <ElemType>
class DlogGroup(typing.Generic[_DlogGroup__ElemType]):
    @staticmethod
    def $init$($this: 'DlogGroup') -> None: ...
    def createRandomElement(self) -> _DlogGroup__ElemType: ...
    def createRandomGenerator(self) -> _DlogGroup__ElemType: ...
    def endExponentiateWithPreComputedValues(self, base: _DlogGroup__ElemType) -> None: ...
    def exponentiate(self, base: _DlogGroup__ElemType, exponent: java.math.BigInteger) -> _DlogGroup__ElemType: ...
    def exponentiateWithPreComputedValues(self, base: _DlogGroup__ElemType, exponent: java.math.BigInteger) -> _DlogGroup__ElemType: ...
    def generator(self) -> _DlogGroup__ElemType: ...
    def identity(self) -> _DlogGroup__ElemType: ...
    def inverseOf(self, groupElement: _DlogGroup__ElemType) -> _DlogGroup__ElemType: ...
    def maxLengthOfByteArrayForEncoding(self) -> int: ...
    def multiplyGroupElements(self, groupElement1: _DlogGroup__ElemType, groupElement2: _DlogGroup__ElemType) -> _DlogGroup__ElemType: ...
    def order(self) -> java.math.BigInteger: ...
    def orderGreaterThan(self, numBits: int) -> bool: ...
    def secureRandom(self) -> java.security.SecureRandom: ...
    def sigmastate$basics$DlogGroup$_setter_$secureRandom_$eq(self, x$1: java.security.SecureRandom) -> None: ...
    def simultaneousMultipleExponentiations(self, groupElements: typing.List[_DlogGroup__ElemType], exponentiations: typing.List[java.math.BigInteger]) -> _DlogGroup__ElemType: ...

class InteractiveParty: ...

_Party__SP = typing.TypeVar('_Party__SP', bound='SigmaProtocol')  # <SP>
_Party__CI = typing.TypeVar('_Party__CI', bound='SigmaProtocolCommonInput')  # <CI>
class Party(typing.Generic[_Party__SP, _Party__CI]):
    def publicInput(self) -> _Party__CI: ...

class ProveDHTuple(sigmastate.SigmaProofOfKnowledgeLeaf['DiffieHellmanTupleProtocol', 'DiffieHellmanTupleProverInput'], scala.Product, scala.Serializable):
    def __init__(self, gv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, hv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, uv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, vv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point): ...
    @staticmethod
    def Code() -> int: ...
    @staticmethod
    def apply(gv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, hv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, uv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, vv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'ProveDHTuple': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, gv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, hv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, uv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, vv: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'ProveDHTuple': ...
    def copy$default$1(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def copy$default$2(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def copy$default$3(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def copy$default$4(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def g(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def gv(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def h(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def hashCode(self) -> int: ...
    def hv(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def opCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def sigmastate$basics$SigmaProtocolCommonInput$_setter_$soundness_$eq(self, x$1: int) -> None: ...
    def size(self) -> int: ...
    def soundness(self) -> int: ...
    def toString(self) -> java.lang.String: ...
    def u(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    @staticmethod
    def unapply(x$0: 'ProveDHTuple') -> scala.Option[scala.Tuple4[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, org.bouncycastle.math.ec.custom.sec.SecP256K1Point, org.bouncycastle.math.ec.custom.sec.SecP256K1Point, org.bouncycastle.math.ec.custom.sec.SecP256K1Point]]: ...
    def uv(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def v(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def vv(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...

class ProveDHTupleProp:
    @staticmethod
    def unapply(p: special.sigma.SigmaProp) -> scala.Option[ProveDHTuple]: ...

class SecP256K1:
    @staticmethod
    def calcK(p: java.math.BigInteger) -> int: ...
    @staticmethod
    def checkCurveMembership(x: java.math.BigInteger, y: java.math.BigInteger) -> bool: ...
    @staticmethod
    def createRandomElement() -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def createRandomGenerator() -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def curve() -> org.bouncycastle.math.ec.ECCurve: ...
    @staticmethod
    def endExponentiateWithPreComputedValues(base: org.bouncycastle.math.ec.ECPoint) -> None: ...
    @staticmethod
    def exponentiate(base: org.bouncycastle.math.ec.ECPoint, exponent: java.math.BigInteger) -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def exponentiateWithPreComputedValues(base: org.bouncycastle.math.ec.ECPoint, exponent: java.math.BigInteger) -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def generator() -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def identity() -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def inverseOf(groupElement: org.bouncycastle.math.ec.ECPoint) -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def k() -> int: ...
    @staticmethod
    def maxLengthOfByteArrayForEncoding() -> int: ...
    @staticmethod
    def multiplyGroupElements(groupElement1: org.bouncycastle.math.ec.ECPoint, groupElement2: org.bouncycastle.math.ec.ECPoint) -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def order() -> java.math.BigInteger: ...
    @staticmethod
    def orderGreaterThan(numBits: int) -> bool: ...
    @staticmethod
    def p() -> java.math.BigInteger: ...
    @staticmethod
    def q() -> java.math.BigInteger: ...
    @staticmethod
    def secureRandom() -> java.security.SecureRandom: ...
    @staticmethod
    def simultaneousMultipleExponentiations(groupElements: typing.List[org.bouncycastle.math.ec.ECPoint], exponentiations: typing.List[java.math.BigInteger]) -> org.bouncycastle.math.ec.ECPoint: ...
    @staticmethod
    def x9params() -> org.bouncycastle.asn1.x9.X9ECParameters: ...

_SigmaProtocol__SP = typing.TypeVar('_SigmaProtocol__SP', bound='SigmaProtocol')  # <SP>
class SigmaProtocol(typing.Generic[_SigmaProtocol__SP]): ...

_SigmaProtocolCommonInput__SP = typing.TypeVar('_SigmaProtocolCommonInput__SP', bound=SigmaProtocol)  # <SP>
class SigmaProtocolCommonInput(typing.Generic[_SigmaProtocolCommonInput__SP]):
    @staticmethod
    def $init$($this: 'SigmaProtocolCommonInput') -> None: ...
    def sigmastate$basics$SigmaProtocolCommonInput$_setter_$soundness_$eq(self, x$1: int) -> None: ...
    def soundness(self) -> int: ...

_SigmaProtocolPrivateInput__SP = typing.TypeVar('_SigmaProtocolPrivateInput__SP', bound=SigmaProtocol)  # <SP>
_SigmaProtocolPrivateInput__CI = typing.TypeVar('_SigmaProtocolPrivateInput__CI', bound=SigmaProtocolCommonInput)  # <CI>
class SigmaProtocolPrivateInput(typing.Generic[_SigmaProtocolPrivateInput__SP, _SigmaProtocolPrivateInput__CI]):
    def publicImage(self) -> _SigmaProtocolPrivateInput__CI: ...

_SigmaProtocolTranscript__SP = typing.TypeVar('_SigmaProtocolTranscript__SP', bound=SigmaProtocol)  # <SP>
_SigmaProtocolTranscript__CI = typing.TypeVar('_SigmaProtocolTranscript__CI', bound=SigmaProtocolCommonInput)  # <CI>
class SigmaProtocolTranscript(typing.Generic[_SigmaProtocolTranscript__SP, _SigmaProtocolTranscript__CI]):
    def a(self) -> 'FirstProverMessage': ...
    def accepted(self) -> bool: ...
    def e(self) -> typing.List[int]: ...
    def x(self) -> _SigmaProtocolTranscript__CI: ...
    def z(self) -> 'SecondProverMessage': ...

_SimulatingProver__SP = typing.TypeVar('_SimulatingProver__SP', bound=SigmaProtocol)  # <SP>
_SimulatingProver__CI = typing.TypeVar('_SimulatingProver__CI', bound=SigmaProtocolCommonInput)  # <CI>
class SimulatingProver(typing.Generic[_SimulatingProver__SP, _SimulatingProver__CI]):
    def challenge(self) -> typing.List[int]: ...

class TranscriptMessage: ...

_ZeroKnowledgeProofOfKnowledge__SP = typing.TypeVar('_ZeroKnowledgeProofOfKnowledge__SP', bound=SigmaProtocol)  # <SP>
class ZeroKnowledgeProofOfKnowledge(typing.Generic[_ZeroKnowledgeProofOfKnowledge__SP]): ...

_BcDlogGroup__ElemType = typing.TypeVar('_BcDlogGroup__ElemType', bound=org.bouncycastle.math.ec.ECPoint)  # <ElemType>
class BcDlogGroup(DlogGroup[_BcDlogGroup__ElemType], typing.Generic[_BcDlogGroup__ElemType]):
    def __init__(self, x9params: org.bouncycastle.asn1.x9.X9ECParameters): ...
    def calcK(self, p: java.math.BigInteger) -> int: ...
    def checkCurveMembership(self, x: java.math.BigInteger, y: java.math.BigInteger) -> bool: ...
    def computeLL(self, groupElements: typing.List[_BcDlogGroup__ElemType], exponentiations: typing.List[java.math.BigInteger]) -> _BcDlogGroup__ElemType: ...
    def computeNaive(self, groupElements: typing.List[_BcDlogGroup__ElemType], exponentiations: typing.List[java.math.BigInteger]) -> _BcDlogGroup__ElemType: ...
    def createRandomElement(self) -> _BcDlogGroup__ElemType: ...
    def createRandomGenerator(self) -> _BcDlogGroup__ElemType: ...
    def curve(self) -> org.bouncycastle.math.ec.ECCurve: ...
    def endExponentiateWithPreComputedValues(self, base: _BcDlogGroup__ElemType) -> None: ...
    def exponentiate(self, base: _BcDlogGroup__ElemType, exponent: java.math.BigInteger) -> _BcDlogGroup__ElemType: ...
    def exponentiateWithPreComputedValues(self, base: _BcDlogGroup__ElemType, exponent: java.math.BigInteger) -> _BcDlogGroup__ElemType: ...
    def generator(self) -> _BcDlogGroup__ElemType: ...
    def identity(self) -> _BcDlogGroup__ElemType: ...
    def inverseOf(self, groupElement: _BcDlogGroup__ElemType) -> _BcDlogGroup__ElemType: ...
    def k(self) -> int: ...
    def maxLengthOfByteArrayForEncoding(self) -> int: ...
    def multiplyGroupElements(self, groupElement1: _BcDlogGroup__ElemType, groupElement2: _BcDlogGroup__ElemType) -> _BcDlogGroup__ElemType: ...
    def order(self) -> java.math.BigInteger: ...
    def orderGreaterThan(self, numBits: int) -> bool: ...
    def p(self) -> java.math.BigInteger: ...
    def q(self) -> java.math.BigInteger: ...
    def secureRandom(self) -> java.security.SecureRandom: ...
    def sigmastate$basics$DlogGroup$_setter_$secureRandom_$eq(self, x$1: java.security.SecureRandom) -> None: ...
    def simultaneousMultipleExponentiations(self, groupElements: typing.List[_BcDlogGroup__ElemType], exponentiations: typing.List[java.math.BigInteger]) -> _BcDlogGroup__ElemType: ...
    def x9params(self) -> org.bouncycastle.asn1.x9.X9ECParameters: ...
    class GroupElementsExponentiations:
        $outer: 'BcDlogGroup' = ...
        def __init__(self, $outer: 'BcDlogGroup', base: _BcDlogGroup__ElemType): ...
        def getExponentiation(self, size: java.math.BigInteger) -> _BcDlogGroup__ElemType: ...
        def two(self) -> java.math.BigInteger: ...

class DiffieHellmanTupleProtocol(SigmaProtocol['DiffieHellmanTupleProtocol']): ...

class DiffieHellmanTupleProverInput(SigmaProtocolPrivateInput[DiffieHellmanTupleProtocol, ProveDHTuple], scala.Product, scala.Serializable):
    def __init__(self, w: java.math.BigInteger, commonInput: ProveDHTuple): ...
    @staticmethod
    def apply(w: java.math.BigInteger, commonInput: ProveDHTuple) -> 'DiffieHellmanTupleProverInput': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def commonInput(self) -> ProveDHTuple: ...
    def copy(self, w: java.math.BigInteger, commonInput: ProveDHTuple) -> 'DiffieHellmanTupleProverInput': ...
    def copy$default$1(self) -> java.math.BigInteger: ...
    def copy$default$2(self) -> ProveDHTuple: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def publicImage(self) -> ProveDHTuple: ...
    @staticmethod
    def random() -> 'DiffieHellmanTupleProverInput': ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'DiffieHellmanTupleProverInput') -> scala.Option[scala.Tuple2[java.math.BigInteger, ProveDHTuple]]: ...
    def w(self) -> java.math.BigInteger: ...

_Prover__SP = typing.TypeVar('_Prover__SP', bound=SigmaProtocol)  # <SP>
_Prover__CI = typing.TypeVar('_Prover__CI', bound=SigmaProtocolCommonInput)  # <CI>
_Prover__PI = typing.TypeVar('_Prover__PI', bound=SigmaProtocolPrivateInput)  # <PI>
class Prover(Party[_Prover__SP, _Prover__CI], typing.Generic[_Prover__SP, _Prover__CI, _Prover__PI]):
    def privateInputOpt(self) -> scala.Option[_Prover__PI]: ...

class ProverMessage(TranscriptMessage): ...

_Verifier__SP = typing.TypeVar('_Verifier__SP', bound=SigmaProtocol)  # <SP>
_Verifier__CI = typing.TypeVar('_Verifier__CI', bound=SigmaProtocolCommonInput)  # <CI>
class Verifier(Party[_Verifier__SP, _Verifier__CI], typing.Generic[_Verifier__SP, _Verifier__CI]):
    @staticmethod
    def $init$($this: 'Verifier') -> None: ...
    def challenge(self) -> typing.List[int]: ...
    def prover(self) -> Prover: ...
    def publicInput(self) -> _Verifier__CI: ...
    def transcript(self) -> scala.concurrent.Future[scala.Option[SigmaProtocolTranscript]]: ...

class VerifierMessage(TranscriptMessage):
    class Challenge$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['VerifierMessage.Challenge.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...

class FirstProverMessage(ProverMessage):
    def bytes(self) -> typing.List[int]: ...

_InteractiveProver__SP = typing.TypeVar('_InteractiveProver__SP', bound=SigmaProtocol)  # <SP>
_InteractiveProver__CI = typing.TypeVar('_InteractiveProver__CI', bound=SigmaProtocolCommonInput)  # <CI>
_InteractiveProver__PI = typing.TypeVar('_InteractiveProver__PI', bound=SigmaProtocolPrivateInput)  # <PI>
class InteractiveProver(Prover[_InteractiveProver__SP, _InteractiveProver__CI, _InteractiveProver__PI], InteractiveParty, typing.Generic[_InteractiveProver__SP, _InteractiveProver__CI, _InteractiveProver__PI]):
    def firstMessage(self) -> FirstProverMessage: ...
    def secondMessage(self, challenge: typing.List[int]) -> 'SecondProverMessage': ...
    def simulate(self, challenge: typing.List[int]) -> scala.Tuple2[FirstProverMessage, 'SecondProverMessage']: ...

_NonInteractiveProver__SP = typing.TypeVar('_NonInteractiveProver__SP', bound=SigmaProtocol)  # <SP>
_NonInteractiveProver__PI = typing.TypeVar('_NonInteractiveProver__PI', bound=SigmaProtocolPrivateInput)  # <PI>
_NonInteractiveProver__CI = typing.TypeVar('_NonInteractiveProver__CI', bound=sigmastate.SigmaProofOfKnowledgeLeaf)  # <CI>
_NonInteractiveProver__P = typing.TypeVar('_NonInteractiveProver__P', bound=sigmastate.UncheckedTree)  # <P>
class NonInteractiveProver(Prover[_NonInteractiveProver__SP, _NonInteractiveProver__CI, _NonInteractiveProver__PI], typing.Generic[_NonInteractiveProver__SP, _NonInteractiveProver__PI, _NonInteractiveProver__CI, _NonInteractiveProver__P]):
    def prove(self, challenge: typing.List[int]) -> _NonInteractiveProver__P: ...

class SecondProverMessage(ProverMessage): ...

class DLogProtocol:
    class DLogInteractiveProver$:
        MODULE$: typing.ClassVar['DLogProtocol.DLogInteractiveProver.'] = ...
        def __init__(self): ...
        def computeCommitment(self, proposition: 'DLogProtocol.ProveDlog', challenge: typing.List[int], secondMessage: 'DLogProtocol.SecondDLogProverMessage') -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
        def firstMessage(self) -> scala.Tuple2[java.math.BigInteger, 'DLogProtocol.FirstDLogProverMessage']: ...
        def secondMessage(self, privateInput: 'DLogProtocol.DLogProverInput', rnd: java.math.BigInteger, challenge: typing.List[int]) -> 'DLogProtocol.SecondDLogProverMessage': ...
        def simulate(self, publicInput: 'DLogProtocol.ProveDlog', challenge: typing.List[int]) -> scala.Tuple2['DLogProtocol.FirstDLogProverMessage', 'DLogProtocol.SecondDLogProverMessage']: ...
    class DLogProverInput(SigmaProtocolPrivateInput['DLogProtocol.DLogSigmaProtocol', 'DLogProtocol.ProveDlog'], scala.Product, scala.Serializable):
        def __init__(self, w: java.math.BigInteger): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def copy(self, w: java.math.BigInteger) -> 'DLogProtocol.DLogProverInput': ...
        def copy$default$1(self) -> java.math.BigInteger: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def publicImage(self) -> 'DLogProtocol.ProveDlog': ...
        def toString(self) -> java.lang.String: ...
        def w(self) -> java.math.BigInteger: ...
    class DLogProverInput$(scala.Serializable):
        MODULE$: typing.ClassVar['DLogProtocol.DLogProverInput.'] = ...
        def __init__(self): ...
        def apply(self, w: java.math.BigInteger) -> 'DLogProtocol.DLogProverInput': ...
        def random(self) -> 'DLogProtocol.DLogProverInput': ...
        def unapply(self, x$0: 'DLogProtocol.DLogProverInput') -> scala.Option[java.math.BigInteger]: ...
    class DLogSigmaProtocol(SigmaProtocol['DLogProtocol.DLogSigmaProtocol']): ...
    class FirstDLogProverMessage(FirstProverMessage, scala.Product, scala.Serializable):
        def __init__(self, ecData: org.bouncycastle.math.ec.custom.sec.SecP256K1Point): ...
        def bytes(self) -> typing.List[int]: ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def copy(self, ecData: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'DLogProtocol.FirstDLogProverMessage': ...
        def copy$default$1(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
        def ecData(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class FirstDLogProverMessage$(scala.runtime.AbstractFunction1[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, 'DLogProtocol.FirstDLogProverMessage'], scala.Serializable):
        MODULE$: typing.ClassVar['DLogProtocol.FirstDLogProverMessage.'] = ...
        def __init__(self): ...
        def apply(self, ecData: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'DLogProtocol.FirstDLogProverMessage': ...
        def toString(self) -> java.lang.String: ...
        def unapply(self, x$0: 'DLogProtocol.FirstDLogProverMessage') -> scala.Option[org.bouncycastle.math.ec.custom.sec.SecP256K1Point]: ...
    class ProveDlog(sigmastate.SigmaProofOfKnowledgeLeaf['DLogProtocol.DLogSigmaProtocol', 'DLogProtocol.DLogProverInput'], scala.Product, scala.Serializable):
        def __init__(self, value: org.bouncycastle.math.ec.custom.sec.SecP256K1Point): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def copy(self, value: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'DLogProtocol.ProveDlog': ...
        def copy$default$1(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def h(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
        def hashCode(self) -> int: ...
        def opCode(self) -> int: ...
        def pkBytes(self) -> typing.List[int]: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def sigmastate$basics$SigmaProtocolCommonInput$_setter_$soundness_$eq(self, x$1: int) -> None: ...
        def size(self) -> int: ...
        def soundness(self) -> int: ...
        def toString(self) -> java.lang.String: ...
        def value(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    class ProveDlog$(scala.Serializable):
        MODULE$: typing.ClassVar['DLogProtocol.ProveDlog.'] = ...
        def __init__(self): ...
        def Code(self) -> int: ...
        def apply(self, value: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'DLogProtocol.ProveDlog': ...
        def unapply(self, x$0: 'DLogProtocol.ProveDlog') -> scala.Option[org.bouncycastle.math.ec.custom.sec.SecP256K1Point]: ...
    class ProveDlogProp$:
        MODULE$: typing.ClassVar['DLogProtocol.ProveDlogProp.'] = ...
        def __init__(self): ...
        def unapply(self, p: special.sigma.SigmaProp) -> scala.Option['DLogProtocol.ProveDlog']: ...
    class SecondDLogProverMessage(SecondProverMessage, scala.Product, scala.Serializable):
        def __init__(self, z: scala.math.BigInt): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def copy(self, z: scala.math.BigInt) -> 'DLogProtocol.SecondDLogProverMessage': ...
        def copy$default$1(self) -> scala.math.BigInt: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
        def z(self) -> scala.math.BigInt: ...
    class SecondDLogProverMessage$(scala.runtime.AbstractFunction1[scala.math.BigInt, 'DLogProtocol.SecondDLogProverMessage'], scala.Serializable):
        MODULE$: typing.ClassVar['DLogProtocol.SecondDLogProverMessage.'] = ...
        def __init__(self): ...
        def apply(self, z: scala.math.BigInt) -> 'DLogProtocol.SecondDLogProverMessage': ...
        def toString(self) -> java.lang.String: ...
        def unapply(self, x$0: 'DLogProtocol.SecondDLogProverMessage') -> scala.Option[scala.math.BigInt]: ...

class FirstDiffieHellmanTupleProverMessage(FirstProverMessage, scala.Product, scala.Serializable):
    def __init__(self, a: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, b: org.bouncycastle.math.ec.custom.sec.SecP256K1Point): ...
    def a(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    @staticmethod
    def apply(a: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, b: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'FirstDiffieHellmanTupleProverMessage': ...
    def b(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, a: org.bouncycastle.math.ec.custom.sec.SecP256K1Point, b: org.bouncycastle.math.ec.custom.sec.SecP256K1Point) -> 'FirstDiffieHellmanTupleProverMessage': ...
    def copy$default$1(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def copy$default$2(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    @staticmethod
    def curried() -> scala.Function1[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, scala.Function1[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, 'FirstDiffieHellmanTupleProverMessage']]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, org.bouncycastle.math.ec.custom.sec.SecP256K1Point], 'FirstDiffieHellmanTupleProverMessage']: ...
    @staticmethod
    def unapply(x$0: 'FirstDiffieHellmanTupleProverMessage') -> scala.Option[scala.Tuple2[org.bouncycastle.math.ec.custom.sec.SecP256K1Point, org.bouncycastle.math.ec.custom.sec.SecP256K1Point]]: ...

class SecondDiffieHellmanTupleProverMessage(SecondProverMessage, scala.Product, scala.Serializable):
    def __init__(self, z: java.math.BigInteger): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['SecondDiffieHellmanTupleProverMessage', _andThen__A]) -> scala.Function1[java.math.BigInteger, _andThen__A]: ...
    @staticmethod
    def apply(z: java.math.BigInteger) -> 'SecondDiffieHellmanTupleProverMessage': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, java.math.BigInteger]) -> scala.Function1[_compose__A, 'SecondDiffieHellmanTupleProverMessage']: ...
    def copy(self, z: java.math.BigInteger) -> 'SecondDiffieHellmanTupleProverMessage': ...
    def copy$default$1(self) -> java.math.BigInteger: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'SecondDiffieHellmanTupleProverMessage') -> scala.Option[java.math.BigInteger]: ...
    def z(self) -> java.math.BigInteger: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("sigmastate.basics")``.

    BcDlogGroup: typing.Type[BcDlogGroup]
    DLogProtocol: typing.Type[DLogProtocol]
    DiffieHellmanTupleInteractiveProver: typing.Type[DiffieHellmanTupleInteractiveProver]
    DiffieHellmanTupleProtocol: typing.Type[DiffieHellmanTupleProtocol]
    DiffieHellmanTupleProverInput: typing.Type[DiffieHellmanTupleProverInput]
    DlogGroup: typing.Type[DlogGroup]
    FirstDiffieHellmanTupleProverMessage: typing.Type[FirstDiffieHellmanTupleProverMessage]
    FirstProverMessage: typing.Type[FirstProverMessage]
    InteractiveParty: typing.Type[InteractiveParty]
    InteractiveProver: typing.Type[InteractiveProver]
    NonInteractiveProver: typing.Type[NonInteractiveProver]
    Party: typing.Type[Party]
    ProveDHTuple: typing.Type[ProveDHTuple]
    ProveDHTupleProp: typing.Type[ProveDHTupleProp]
    Prover: typing.Type[Prover]
    ProverMessage: typing.Type[ProverMessage]
    SecP256K1: typing.Type[SecP256K1]
    SecondDiffieHellmanTupleProverMessage: typing.Type[SecondDiffieHellmanTupleProverMessage]
    SecondProverMessage: typing.Type[SecondProverMessage]
    SigmaProtocol: typing.Type[SigmaProtocol]
    SigmaProtocolCommonInput: typing.Type[SigmaProtocolCommonInput]
    SigmaProtocolPrivateInput: typing.Type[SigmaProtocolPrivateInput]
    SigmaProtocolTranscript: typing.Type[SigmaProtocolTranscript]
    SimulatingProver: typing.Type[SimulatingProver]
    TranscriptMessage: typing.Type[TranscriptMessage]
    Verifier: typing.Type[Verifier]
    VerifierMessage: typing.Type[VerifierMessage]
    ZeroKnowledgeProofOfKnowledge: typing.Type[ZeroKnowledgeProofOfKnowledge]



import java.lang
import scala
import scala.collection
import scala.math
import scala.util
import scorex.crypto.authds.avltree
import scorex.crypto.authds.avltree.batch
import scorex.crypto.authds.legacy
import scorex.crypto.authds.legacy.treap
import scorex.crypto.authds.merkle
import scorex.util
import scorex.util.encode
import supertagged
import typing



class Direction: ...

class GoingLeft:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class GoingRight:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class LeafFound:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class LeafNotFound:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class ProofIterator:
    @staticmethod
    def $init$($this: 'ProofIterator') -> None: ...
    def dequeueBalance(self) -> int: ...
    def dequeueDirection(self) -> Direction: ...
    def dequeueKey(self) -> typing.List[int]: ...
    def dequeueLeftLabel(self) -> typing.List[int]: ...
    def dequeueLevel(self) -> scorex.crypto.authds.legacy.treap.Level: ...
    def dequeueNextLeafKey(self) -> typing.List[int]: ...
    def dequeueRightLabel(self) -> typing.List[int]: ...
    def dequeueValue(self) -> typing.List[int]: ...
    def initializeIterator(self) -> None: ...
    def proofSeq(self) -> scala.collection.Seq['TwoPartyProofElement']: ...
    def scorex$crypto$authds$ProofIterator$$i(self) -> int: ...
    def scorex$crypto$authds$ProofIterator$$i_$eq(self, x$1: int) -> None: ...

class TwoPartyDictionary(scorex.util.ScorexEncoding):
    def rootHash(self) -> typing.List[int]: ...
    _run__O = typing.TypeVar('_run__O', bound=scorex.crypto.authds.avltree.batch.Operation)  # <O>
    def run(self, operation: _run__O) -> scala.util.Try['TwoPartyProof']: ...

class TwoPartyProofElement(scorex.util.ScorexEncoding):
    def bytes(self) -> typing.List[int]: ...

class package:
    @staticmethod
    def EmptyByteArray() -> typing.List[int]: ...
    class ADDigest$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.ADDigest.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class ADKey$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.ADKey.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class ADValue$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.ADValue.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class Balance$(supertagged.package.TaggedType[typing.Any]):
        MODULE$: typing.ClassVar['package.Balance.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.Any, supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.Any, supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.Any, supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.Any]) -> scala.math.Ordering[typing.Any]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.Any, supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class LeafData$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.LeafData.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class SerializedAdProof$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.SerializedAdProof.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class Side$(supertagged.package.TaggedType[typing.Any]):
        MODULE$: typing.ClassVar['package.Side.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.Any, supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.Any, supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.Any, supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.Any]) -> scala.math.Ordering[typing.Any]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.Any, supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...

class AVLProofElement(TwoPartyProofElement): ...

class SLTProofElement(TwoPartyProofElement): ...

class TwoPartyProof(ProofIterator):
    @staticmethod
    def $init$($this: 'TwoPartyProof') -> None: ...
    def key(self) -> typing.List[int]: ...
    def proofSeq(self) -> scala.collection.Seq[TwoPartyProofElement]: ...
    @typing.overload
    def verify(self, digest: typing.List[int], updateFn: scala.Function1[scala.Option[typing.List[int]], scala.util.Try[scala.Option[typing.List[int]]]]) -> scala.Option[typing.List[int]]: ...
    @typing.overload
    def verify(self, digest: typing.List[int], modification: scorex.crypto.authds.avltree.batch.Modification) -> scala.Option[typing.List[int]]: ...

class WTProofElement(TwoPartyProofElement): ...

class Key(WTProofElement, AVLProofElement, SLTProofElement):
    @staticmethod
    def $init$($this: 'Key') -> None: ...
    def bytes(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def scorex$crypto$authds$Key$_setter_$bytes_$eq(self, x$1: typing.List[int]) -> None: ...
    def toString(self) -> java.lang.String: ...

class ProofBalance(AVLProofElement, scala.Product, scala.Serializable):
    def __init__(self, e: int): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofBalance', _andThen__A]) -> scala.Function1[typing.Any, _andThen__A]: ...
    @staticmethod
    def apply(e: int) -> 'ProofBalance': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.Any]) -> scala.Function1[_compose__A, 'ProofBalance']: ...
    def copy(self, e: int) -> 'ProofBalance': ...
    def copy$default$1(self) -> int: ...
    def e(self) -> int: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofBalance') -> scala.Option[typing.Any]: ...

class ProofDirection(WTProofElement, AVLProofElement, scala.Product, scala.Serializable):
    def __init__(self, direction: Direction): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofDirection', _andThen__A]) -> scala.Function1[Direction, _andThen__A]: ...
    @staticmethod
    def apply(direction: Direction) -> 'ProofDirection': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, Direction]) -> scala.Function1[_compose__A, 'ProofDirection']: ...
    def copy(self, direction: Direction) -> 'ProofDirection': ...
    def copy$default$1(self) -> Direction: ...
    def direction(self) -> Direction: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def isLeaf(self) -> bool: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofDirection') -> scala.Option[Direction]: ...

class ProofLabel(WTProofElement, AVLProofElement, SLTProofElement):
    @staticmethod
    def $init$($this: 'ProofLabel') -> None: ...
    def bytes(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def scorex$crypto$authds$ProofLabel$_setter_$bytes_$eq(self, x$1: typing.List[int]) -> None: ...
    def toString(self) -> java.lang.String: ...

class ProofLevel(WTProofElement, AVLProofElement, SLTProofElement, scala.Product, scala.Serializable):
    def __init__(self, e: scorex.crypto.authds.legacy.treap.Level): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofLevel', _andThen__A]) -> scala.Function1[scorex.crypto.authds.legacy.treap.Level, _andThen__A]: ...
    @staticmethod
    def apply(e: scorex.crypto.authds.legacy.treap.Level) -> 'ProofLevel': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, scorex.crypto.authds.legacy.treap.Level]) -> scala.Function1[_compose__A, 'ProofLevel']: ...
    def copy(self, e: scorex.crypto.authds.legacy.treap.Level) -> 'ProofLevel': ...
    def copy$default$1(self) -> scorex.crypto.authds.legacy.treap.Level: ...
    def e(self) -> scorex.crypto.authds.legacy.treap.Level: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofLevel') -> scala.Option[scorex.crypto.authds.legacy.treap.Level]: ...

class ProofValue(WTProofElement, AVLProofElement, SLTProofElement, scala.Product, scala.Serializable):
    def __init__(self, e: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofValue', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(e: typing.List[int]) -> 'ProofValue': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'ProofValue']: ...
    def copy(self, e: typing.List[int]) -> 'ProofValue': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofValue') -> scala.Option[typing.List[int]]: ...

class ProofKey(Key, scala.Product, scala.Serializable):
    def __init__(self, e: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofKey', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(e: typing.List[int]) -> 'ProofKey': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'ProofKey']: ...
    def copy(self, e: typing.List[int]) -> 'ProofKey': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$crypto$authds$Key$_setter_$bytes_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofKey') -> scala.Option[typing.List[int]]: ...

class ProofLeftLabel(ProofLabel, scala.Product, scala.Serializable):
    def __init__(self, e: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofLeftLabel', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(e: typing.List[int]) -> 'ProofLeftLabel': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'ProofLeftLabel']: ...
    def copy(self, e: typing.List[int]) -> 'ProofLeftLabel': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$crypto$authds$ProofLabel$_setter_$bytes_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofLeftLabel') -> scala.Option[typing.List[int]]: ...

class ProofNextLeafKey(Key, scala.Product, scala.Serializable):
    def __init__(self, e: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofNextLeafKey', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(e: typing.List[int]) -> 'ProofNextLeafKey': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'ProofNextLeafKey']: ...
    def copy(self, e: typing.List[int]) -> 'ProofNextLeafKey': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$crypto$authds$Key$_setter_$bytes_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofNextLeafKey') -> scala.Option[typing.List[int]]: ...

class ProofRightLabel(ProofLabel, scala.Product, scala.Serializable):
    def __init__(self, e: typing.List[int]): ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['ProofRightLabel', _andThen__A]) -> scala.Function1[typing.List[int], _andThen__A]: ...
    @staticmethod
    def apply(e: typing.List[int]) -> 'ProofRightLabel': ...
    def bytes(self) -> typing.List[int]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.List[int]]) -> scala.Function1[_compose__A, 'ProofRightLabel']: ...
    def copy(self, e: typing.List[int]) -> 'ProofRightLabel': ...
    def copy$default$1(self) -> typing.List[int]: ...
    def e(self) -> typing.List[int]: ...
    def encoder(self) -> scorex.util.encode.BytesEncoder: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def scorex$crypto$authds$ProofLabel$_setter_$bytes_$eq(self, x$1: typing.List[int]) -> None: ...
    def scorex$util$ScorexEncoding$_setter_$encoder_$eq(self, x$1: scorex.util.encode.BytesEncoder) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'ProofRightLabel') -> scala.Option[typing.List[int]]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto.authds")``.

    AVLProofElement: typing.Type[AVLProofElement]
    Direction: typing.Type[Direction]
    GoingLeft: typing.Type[GoingLeft]
    GoingRight: typing.Type[GoingRight]
    Key: typing.Type[Key]
    LeafFound: typing.Type[LeafFound]
    LeafNotFound: typing.Type[LeafNotFound]
    ProofBalance: typing.Type[ProofBalance]
    ProofDirection: typing.Type[ProofDirection]
    ProofIterator: typing.Type[ProofIterator]
    ProofKey: typing.Type[ProofKey]
    ProofLabel: typing.Type[ProofLabel]
    ProofLeftLabel: typing.Type[ProofLeftLabel]
    ProofLevel: typing.Type[ProofLevel]
    ProofNextLeafKey: typing.Type[ProofNextLeafKey]
    ProofRightLabel: typing.Type[ProofRightLabel]
    ProofValue: typing.Type[ProofValue]
    SLTProofElement: typing.Type[SLTProofElement]
    TwoPartyDictionary: typing.Type[TwoPartyDictionary]
    TwoPartyProof: typing.Type[TwoPartyProof]
    TwoPartyProofElement: typing.Type[TwoPartyProofElement]
    WTProofElement: typing.Type[WTProofElement]
    package: typing.Type[package]
    avltree: scorex.crypto.authds.avltree.__module_protocol__
    legacy: scorex.crypto.authds.legacy.__module_protocol__
    merkle: scorex.crypto.authds.merkle.__module_protocol__
